@@ -201,12 +201,18 @@ std::string cw::find_winner (float *inbuf, int twodots)
 			}
 	}
 
+	std::string sc;
 	if (!som_table[winner].rpr.empty()) {
-		string sc = morse.rx_lookup(som_table[winner].rpr);
-		if (sc.empty()) return "*";
-		else return sc;
+		sc = morse.rx_lookup(som_table[winner].rpr);
+		if (sc.empty()) 
+			sc = (progdefaults.CW_noise == '*' ? "*" :
+				  progdefaults.CW_noise == '_' ? "_" :
+				  progdefaults.CW_noise == ' ' ? " " : "");
 	} else
-		return "*";
+		sc = (progdefaults.CW_noise == '*' ? "*" :
+			  progdefaults.CW_noise == '_' ? "_" :
+			  progdefaults.CW_noise == ' ' ? " " : "");
+	return sc;
 }
 
 void cw::tx_init()
@@ -300,7 +306,7 @@ cw::cw() : modem()
 	samplerate = CW_SAMPLERATE;
 	fragmentsize = CWMaxSymLen;
 
-	cw_speed  = progdefaults.CWspeed;
+	wpm = cw_speed  = progdefaults.CWspeed;
 	bandwidth = progdefaults.CWbandwidth;
 
 	cw_send_speed = cw_speed;
@@ -425,13 +431,13 @@ void cw::reset_rx_filter()
 
 void cw::sync_transmit_parameters()
 {
-	wpm = usedefaultWPM ? progdefaults.defCWspeed : progdefaults.CWspeed;
+//	wpm = usedefaultWPM ? progdefaults.defCWspeed : progdefaults.CWspeed;
 	fwpm = progdefaults.CWfarnsworth;
 
 	cw_send_dot_length = KWPM / progdefaults.CWspeed;
 	cw_send_dash_length = 3 * cw_send_dot_length;
 
-	nusymbollen = (int)round(samplerate * 1.2 / wpm);
+	nusymbollen = (int)round(samplerate * 1.2 / progdefaults.CWspeed);
 	nufsymlen = (int)round(samplerate * 1.2 / fwpm);
 
 	if (symbollen != nusymbollen ||
@@ -448,6 +454,8 @@ void cw::sync_transmit_parameters()
 
 void cw::sync_parameters()
 {
+	sync_transmit_parameters();
+
 	if (use_nanoIO) {
 		if (nano_wpm != progdefaults.CWspeed) {
 			nano_wpm = progdefaults.CWspeed;
@@ -459,8 +467,6 @@ void cw::sync_parameters()
 		}
 	}
 
-	sync_transmit_parameters();
-
 // check if user changed the tracking or the cw default speed
 	if ((cwTrack != progdefaults.CWtrack) ||
 		(cw_send_speed != progdefaults.CWspeed)) {
@@ -518,16 +524,10 @@ static int max_dash = 3 * KWPM / 5;
 	sync_parameters();
 }
 
-//=======================================================================
-//update_syncscope()
-//Routine called to update the display on the sync scope display.
-//For CW this is an o scope pattern that shows the cw data stream.
-//=======================================================================
-
 void cw::update_Status()
 {
 	put_MODEstatus("CW %s Rx %d", usedefaultWPM ? "*" : " ", cw_receive_speed);
-
+	REQ(set_CWwpm);
 }
 
 //=======================================================================
@@ -882,7 +882,10 @@ int cw::handle_event(int cw_event, string &sc)
 			sc = morse.rx_lookup(rx_rep_buf);
 			if (sc.empty()) {
 // invalid decode... let user see error
-				sc = "*";
+			sc = (progdefaults.CW_noise == '*' ? "*" :
+				  progdefaults.CW_noise == '_' ? "_" :
+				  progdefaults.CW_noise == ' ' ? " " : "");
+
 			}
 			rx_rep_buf.clear();
 			cw_receive_state = RS_IDLE;
@@ -1151,6 +1154,17 @@ int cw::tx_process()
 		return 0;
 	}
 
+	if (progdefaults.use_FLRIGkeying) {
+		if (c == GET_TX_CHAR_ETX || stopflag) {
+			stopflag = false;
+			put_echo_char('\n');
+			return -1;
+		}
+		flrig_cwio_send(c);
+		put_echo_char(c);
+		return 0;
+	}
+
 	if (progStatus.WK_online) {
 		if (c == GET_TX_CHAR_ETX || stopflag) {
 			stopflag = false;
@@ -1257,12 +1271,18 @@ void cw::decWPM()
 void cw::toggleWPM()
 {
 	usedefaultWPM = !usedefaultWPM;
+	if (usedefaultWPM) {
+		wpm = progdefaults.CWspeed;
+		progdefaults.CWspeed = progdefaults.defCWspeed;
+	} else {
+		progdefaults.CWspeed = wpm;
+	}
 	sync_parameters();
 	update_Status();
 }
 
 // ---------------------------------------------------------------------
-// TTY output on DTR/RTS signal lines
+// CW output on DTR/RTS signal lines
 //----------------------------------------------------------------------
 
 Cserial CW_KEYLINE_serial;
@@ -1333,6 +1353,46 @@ static cMorse *cwio_morse = 0;
 static queue<int> fifo;
 static std::string cwio_prosigns;
 
+//----------------------------------------------------------------------
+// CW output using flrig cwio calls
+//----------------------------------------------------------------------
+static char lastcwiochar = 0;
+void flrig_cwio_send(char c)
+{
+	if (cwio_morse == 0) {
+		cwio_morse = new cMorse;
+		cwio_morse->init();
+	}
+
+	if (c == '[') {
+		flrig_cwio_ptt(1);
+		return;
+	}
+	if (c == ']') {
+		flrig_cwio_ptt(0);
+		return;
+	}
+	std::string s = " ";
+	s[0] = c;
+	flrig_cwio_send_text(s);
+
+	int tc = 1200 / progdefaults.CWspeed;
+	if (progdefaults.CWusefarnsworth && (progdefaults.CWspeed > progdefaults.CWfarnsworth))
+		tc = 1200 / progdefaults.CWfarnsworth;
+
+	if (c == ' ') {
+		if (lastcwiochar == ' ')
+			tc *= 7;
+		else
+		tc *= 5;
+	} else
+		tc *= (cwio_morse->tx_length(c));
+	lastcwiochar = c;
+	MilliSleep(tc);
+}
+
+//----------------------------------------------------------------------
+
 void cwio_key(int on)
 {
 	if (CW_KEYLINE_isopen ||
@@ -1414,7 +1474,8 @@ void send_cwio(int c)
 	}
 
 	string code;
-	code = cwio_morse->tx_lookup(c);	if (!code.length()) {
+	code = cwio_morse->tx_lookup(c);
+	if (!code.length()) {
 		return;
 	}
 
